@@ -34,12 +34,34 @@ def _interactive_display(filename):
     return in_notebook
 
 
-def _make_pretty_ticks(x_min, x_max, x_axis_is_time):
+def _check_equally_spaced(xs):
+    unique_xs = list(set(xi for _x in xs for xi in _x))
+    unique_xs.sort()
+    diffs = set()
+    for _i, xi in enumerate(unique_xs[:-1]):
+        diff = unique_xs[_i + 1] - xi
+        diffs.add(diff)
 
-    if x_axis_is_time:
-        x_ticks = [datetime.fromtimestamp(_xt) for _xt in [x_min, x_max]]
+    return len(diffs) == 1, unique_xs
+
+
+def _make_pretty_ticks(x_min, x_max, axis_is_time, xs):
+
+    equally_spaced, unique_xs = _check_equally_spaced(xs)
+    if axis_is_time:
+        if len(unique_xs) <= 12:
+            # if small number of x, just use them
+            x_ticks_timestamps = unique_xs
+        else:
+            x_ticks_timestamps = [x_min, x_max]
+
+        x_ticks = [datetime.fromtimestamp(_xt) for _xt in x_ticks_timestamps]
     else:
-        # TODO: if small number of x and equally spaced, just use them
+        if len(unique_xs) <= 10 and equally_spaced:
+            # if small number of x and equally spaced, just use them
+            x_ticks = unique_xs
+            return x_ticks
+
         min_rounded = '{:.1e}'.format(x_min)
         max_rounded = '{:.1e}'.format(x_max)
 
@@ -54,8 +76,23 @@ def _make_pretty_ticks(x_min, x_max, x_axis_is_time):
                 min_out = float(str(-.1) + 'e' + exp_st) + float(min_rounded)
                 x_ticks.insert(0, min_out)
         else:
-            # TODO: add in more ticks if available
+            # add in more ticks if available
+            mean = (float(max_rounded) + float(min_rounded)) / 2.
+            thrd = (float(max_rounded) + float(min_rounded)) / 3.
+            if float('{:.1e}'.format(mean)) == mean:
+                x_ticks.append(float(mean))
+                l_mean = (mean + float(min_rounded)) / 2.
+                u_mean = (mean + float(max_rounded)) / 2.
+                if float('{:.1e}'.format(l_mean)) == l_mean:
+                    if float('{:.1e}'.format(u_mean)) == u_mean:
+                        x_ticks.append(float(l_mean))
+                        x_ticks.append(float(u_mean))
+            elif float('{:.1e}'.format(thrd)) == thrd:
+                if float('{:.1e}'.format(2*thrd)) == 2*thrd:
+                    x_ticks.append(float(thrd))
+                    x_ticks.append(2*float(thrd))
             x_ticks.append(float(max_rounded))
+            x_ticks.sort()
 
     return x_ticks
 
@@ -83,6 +120,61 @@ def _num2pretty_string(x_tick):
     return x_out
 
 
+def find_safe_time_trunc(x_ticks):
+    unique_year = set()
+    unique_month = set()
+    unique_day = set()
+    unique_hour = set()
+    unique_minute = set()
+    unique_second = set()
+    unique_microsecond = set()
+    for xt in x_ticks:
+        unique_year.add(xt.year)
+        unique_month.add(xt.month)
+        unique_day.add(xt.day)
+        unique_hour.add(xt.hour)
+        unique_minute.add(xt.minute)
+        unique_second.add(xt.second)
+        unique_microsecond.add(xt.microsecond)
+
+    all_same = [len(unique_year) == 1, len(unique_month) == 1,
+                len(unique_day) == 1, len(unique_hour) == 1,
+                len(unique_minute) == 1, len(unique_second) == 1,
+                len(unique_microsecond) == 1]
+
+    date_part = ['year', 'month', 'day', 'hour', 'minute', 'second', 'msecond']
+    max_t_trunc = date_part[all_same.index(False)]
+    min_t_trunc = date_part[::-1][all_same[::-1].index(False)]
+
+    return max_t_trunc, min_t_trunc
+
+
+def _tm2pretty_string(x_tick, max_t_trunc, min_t_trunc, n_ticks, isfirst):
+
+    if n_ticks < 6 and min_t_trunc in ('year', 'month', 'day'):
+        x_out = str(x_tick).split(' ')[0]
+    elif min_t_trunc == 'year':
+        x_out = str(x_tick.year)
+    elif min_t_trunc == 'month' and max_t_trunc=='year':
+        x_out = str(x_tick.year) + '-' + str(x_tick.month)
+    elif min_t_trunc == 'month' and max_t_trunc=='month':
+        x_out = x_tick.strftime("%B")[:3]
+        if isfirst:
+            x_out = str(x_tick.year) + '-' + x_tick.strftime("%B")[:3]
+    elif min_t_trunc =='day':
+        x_out = str(x_tick).split(' ')[0]
+    elif max_t_trunc in ('hour', 'minute', 'second', 'msecond'):
+        x_out = str(x_tick).split(' ')[1]
+    elif min_t_trunc in ('hour', 'minute'):
+        x_out = ':'.join(str(x_tick).split(':')[0:2])
+        if max_t_trunc in ('year', 'month', 'day'):
+            x_out = '-'.join(x_out.split('-')[1:])
+    else:
+        x_out = str(x_tick)
+
+    return x_out
+
+
 def _remove_extra_whitespace(in_string):
     in_string = ' '.join(in_string.split())
     return in_string
@@ -415,18 +507,21 @@ def base_plot(xs, ys, yus=None, yls=None, filename='plot.svg',
     y_max = max([max(y) for y in all_ys])
 
     if x_ticks is None:
-        x_ticks = _make_pretty_ticks(x_min, x_max, x_axis_is_time)
+        x_ticks = _make_pretty_ticks(x_min, x_max, x_axis_is_time, all_xs)
 
     if not x_axis_is_time and x_ticks_text is None:
         x_ticks_text = [_num2pretty_string(_xt) for _xt in x_ticks]
     if x_axis_is_time and len(x_ticks) >= 2:
         if x_ticks_text is None:
-            x_ticks_text = [str(_xt).split(' ')[0] for _xt in x_ticks]
+            max_t_trunc, min_t_trunc = find_safe_time_trunc(x_ticks)
+            x_ticks_text = [_tm2pretty_string(_xt, max_t_trunc, min_t_trunc,
+                                              len(x_ticks), _xt==x_ticks[0])
+                            for _xt in x_ticks]
         # trying to convert ticks
         x_ticks = [datetime.timestamp(_xt) for _xt in x_ticks]
 
     if y_ticks is None:
-        y_ticks = _make_pretty_ticks(y_min, y_max, False)
+        y_ticks = _make_pretty_ticks(y_min, y_max, False, all_ys)
 
     if type(labels) == str and len(ys) == 1:
         labels = [labels]  # convert to list with one string
